"""
Модуль для сканирования сети
на наличие расшаренных(общедоступных)
директорий и дисков локальной сети.
"""

from os import popen
from shutil import which
from subprocess import check_output, CalledProcessError


class SMBscan:
    def __init__(self, ip, user_pass):
        """
        Инициализация класса для сканирования компьютера на наличие
        общедоступных директорий и дисков.
        :param ip: IP-адрес для сканирования.
        :param user_pass: связка логин пароль, которая запрашивается у пользователя и
        передается в виде login%pass. Если пользователь не вводит логин-пароль, по умолчанию
        устанавливается значение -N, что означает отсутствие пароля.
        """
        self.ip = ip
        if user_pass == "" or user_pass == " ":
            self.user_pass = "-N"
        else:
            self.user_pass = user_pass

    def smb_get(self):
        """
        Сканирование компьютера с использованием smbclient. Получаем вывод команды с параметрами,
        где: -L - опция для просмотра доступных сервисов компьютера.
        :return: возвращает список общедоступных ресурсов.
        """
        try:
            view = check_output(f'{which("smbclient")} -L {self.ip} '
                                           f'{self.user_pass} 2>/dev/null', shell=True).decode().splitlines()
            view_smb = "\n".join([x.strip() for x in view if not x.startswith("SMB")])
            return view_smb
        except CalledProcessError:
            return 'Ошибка соединения'

    def smb_access(self):
        """
        Для начала функция получает данные о доступных сервисах, после чего, данные фильтруются
        и выбираются только диски, для которых выполняется попытка чтения содержимого.
        :return: возвращает список найденных ресурсов.
        """
        disks = check_output(f'{which("smbclient")} -L '
                                        f'{self.ip} {self.user_pass} '
                                        f'2>/dev/null| grep " Disk " | sed -e "s/ Disk .*//"', shell=True).decode()
        disks = [d.strip() for d in disks.splitlines()]
        disks_list = dict()
        for disk in disks:
            f = popen(f'{which("smbclient")} //{self.ip}/"{disk}" {self.user_pass} -c "dir;exit" 2>/dev/null').readlines()
            d = [x.strip() for x in f]
            disks_list.update({disk: d})
        if disks_list:
            found = ''
            for key in disks_list:
                found = found + "Найден общий ресурс: %s\n%s\n\n" % (key, "\n".join(disks_list[key]))
            found = found.replace("NT_STATUS_ACCESS_DENIED", "Общий ресурс виден, но защищен паролем"). \
                replace("NT_STATUS_OBJECT_PATH_NOT_FOUND", "Не удается получить доступ к файлу"). \
                replace("NT_STATUS_NO_MEDIA_IN_DEVICE", "Диск общий, но нет доступа к носителю"). \
                replace("NT_STATUS_DEVICE_DATA_ERROR", "Не удается получить доступ к общему ресурсу")
            return "\n" + found
        else:
            return 'Не могу получить доступ'
